@@ -18,9 +18,9 @@
 use clap::Parser;
 use datafusion::error::Result;
 use datafusion::execution::context::SessionConfig;
+use datafusion::prelude::SessionContext;
 use datafusion_cli::{
-    context::Context, exec, print_format::PrintFormat, print_options::PrintOptions,
-    DATAFUSION_CLI_VERSION,
+    exec, print_format::PrintFormat, print_options::PrintOptions, DATAFUSION_CLI_VERSION,
 };
 use mimalloc::MiMalloc;
 use std::env;
@@ -92,13 +92,13 @@ pub async fn main() -> Result<()> {
         env::set_current_dir(&p).unwrap();
     };
 
-    let mut session_config = SessionConfig::new().with_information_schema(true);
+    let mut session_config = SessionConfig::from_env().with_information_schema(true);
 
     if let Some(batch_size) = args.batch_size {
         session_config = session_config.with_batch_size(batch_size);
     };
 
-    let mut ctx: Context = Context::new_local(&session_config);
+    let mut ctx = SessionContext::with_config(session_config.clone());
 
     let mut print_options = PrintOptions {
         format: args.format,
