@@ -36,17 +36,17 @@ use crate::arrow::error::ArrowError;
 use crate::arrow::record_batch::RecordBatch;
 use crate::arrow::util::bit_util;
 use crate::error::{DataFusionError, Result};
-use arrow::array::{BinaryArray, GenericListArray};
-use arrow::datatypes::SchemaRef;
-use arrow::error::ArrowError::SchemaError;
-use arrow::error::Result as ArrowResult;
-use avro_rs::{
+use apache_avro::{
     schema::{Schema as AvroSchema, SchemaKind},
     types::Value,
     AvroResult, Error as AvroError, Reader as AvroReader,
 };
+use arrow::array::{BinaryArray, GenericListArray};
+use arrow::datatypes::SchemaRef;
+use arrow::error::ArrowError::SchemaError;
+use arrow::error::Result as ArrowResult;
 use num_traits::NumCast;
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use std::io::Read;
 use std::sync::Arc;
 
@@ -56,7 +56,7 @@ pub struct AvroArrowArrayReader<'a, R: Read> {
     reader: AvroReader<'a, R>,
     schema: SchemaRef,
     projection: Option<Vec<String>>,
-    schema_lookup: HashMap<String, usize>,
+    schema_lookup: BTreeMap<String, usize>,
 }
 
 impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
@@ -76,7 +76,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
         })
     }
 
-    pub fn schema_lookup(schema: AvroSchema) -> Result<HashMap<String, usize>> {
+    pub fn schema_lookup(schema: AvroSchema) -> Result<BTreeMap<String, usize>> {
         match schema {
             AvroSchema::Record {
                 lookup: ref schema_lookup,
@@ -101,12 +101,10 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                     "Failed to parse avro value: {:?}",
                     e
                 ))),
-                other => {
-                    return Err(ArrowError::ParseError(format!(
-                        "Row needs to be of type object, got: {:?}",
-                        other
-                    )))
-                }
+                other => Err(ArrowError::ParseError(format!(
+                    "Row needs to be of type object, got: {:?}",
+                    other
+                ))),
             })
             .collect::<ArrowResult<Vec<Vec<(String, Value)>>>>()?;
         if rows.is_empty() {
@@ -114,7 +112,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
             return Ok(None);
         }
         let rows = rows.iter().collect::<Vec<&Vec<(String, Value)>>>();
-        let projection = self.projection.clone().unwrap_or_else(Vec::new);
+        let projection = self.projection.clone().unwrap_or_default();
         let arrays =
             self.build_struct_array(rows.as_slice(), self.schema.fields(), &projection);
         let projected_fields: Vec<Field> = if projection.is_empty() {
@@ -122,8 +120,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
         } else {
             projection
                 .iter()
-                .map(|name| self.schema.column_with_name(name))
-                .flatten()
+                .filter_map(|name| self.schema.column_with_name(name))
                 .map(|(_, field)| field.clone())
                 .collect()
         };
@@ -131,58 +128,52 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
         arrays.and_then(|arr| RecordBatch::try_new(projected_schema, arr).map(Some))
     }
 
-    fn build_boolean_array(
-        &self,
-        rows: RecordSlice,
-        col_name: &str,
-    ) -> ArrowResult<ArrayRef> {
+    fn build_boolean_array(&self, rows: RecordSlice, col_name: &str) -> ArrayRef {
         let mut builder = BooleanBuilder::new(rows.len());
         for row in rows {
             if let Some(value) = self.field_lookup(col_name, row) {
-                if let Some(boolean) = resolve_boolean(&value) {
-                    builder.append_value(boolean)?
+                if let Some(boolean) = resolve_boolean(value) {
+                    builder.append_value(boolean)
                 } else {
-                    builder.append_null()?;
+                    builder.append_null();
                 }
             } else {
-                builder.append_null()?;
+                builder.append_null();
             }
         }
-        Ok(Arc::new(builder.finish()))
+        Arc::new(builder.finish())
     }
 
-    #[allow(clippy::unnecessary_wraps)]
     fn build_primitive_array<T: ArrowPrimitiveType + Resolver>(
         &self,
         rows: RecordSlice,
         col_name: &str,
-    ) -> ArrowResult<ArrayRef>
+    ) -> ArrayRef
     where
         T: ArrowNumericType,
         T::Native: num_traits::cast::NumCast,
     {
-        Ok(Arc::new(
+        Arc::new(
             rows.iter()
                 .map(|row| {
                     self.field_lookup(col_name, row)
-                        .and_then(|value| resolve_item::<T>(&value))
+                        .and_then(|value| resolve_item::<T>(value))
                 })
                 .collect::<PrimitiveArray<T>>(),
-        ))
+        )
     }
 
     #[inline(always)]
-    #[allow(clippy::unnecessary_wraps)]
     fn build_string_dictionary_builder<T>(
         &self,
         row_len: usize,
-    ) -> ArrowResult<StringDictionaryBuilder<T>>
+    ) -> StringDictionaryBuilder<T>
     where
         T: ArrowPrimitiveType + ArrowDictionaryKeyType,
     {
         let key_builder = PrimitiveBuilder::<T>::new(row_len);
         let values_builder = StringBuilder::new(row_len * 5);
-        Ok(StringDictionaryBuilder::new(key_builder, values_builder))
+        StringDictionaryBuilder::new(key_builder, values_builder)
     }
 
     fn build_wrapped_list_array(
@@ -272,7 +263,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
             }
             DataType::Dictionary(_, _) => {
                 let values_builder =
-                    self.build_string_dictionary_builder::<D>(rows.len() * 5)?;
+                    self.build_string_dictionary_builder::<D>(rows.len() * 5);
                 Box::new(ListBuilder::new(values_builder))
             }
             e => {
@@ -290,7 +281,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                     vec![Some(v.to_string())]
                 } else if let Value::Array(n) = value {
                     n.iter()
-                        .map(|v| resolve_string(&v))
+                        .map(resolve_string)
                         .collect::<ArrowResult<Vec<String>>>()?
                         .into_iter()
                         .map(Some)
@@ -298,7 +289,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                 } else if let Value::Null = value {
                     vec![None]
                 } else if !matches!(value, Value::Record(_)) {
-                    vec![Some(resolve_string(&value)?)]
+                    vec![Some(resolve_string(value)?)]
                 } else {
                     return Err(SchemaError(
                         "Only scalars are currently supported in Avro arrays".to_string(),
@@ -317,14 +308,14 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                             ))?;
                         for val in vals {
                             if let Some(v) = val {
-                                builder.values().append_value(&v)?
+                                builder.values().append_value(&v)
                             } else {
-                                builder.values().append_null()?
+                                builder.values().append_null()
                             };
                         }
 
                         // Append to the list
-                        builder.append(true)?;
+                        builder.append(true);
                     }
                     DataType::Dictionary(_, _) => {
                         let builder = builder.as_any_mut().downcast_mut::<ListBuilder<StringDictionaryBuilder<D>>>().ok_or_else(||ArrowError::SchemaError(
@@ -334,12 +325,12 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                             if let Some(v) = val {
                                 let _ = builder.values().append(&v)?;
                             } else {
-                                builder.values().append_null()?
+                                builder.values().append_null()
                             };
                         }
 
                         // Append to the list
-                        builder.append(true)?;
+                        builder.append(true);
                     }
                     e => {
                         return Err(SchemaError(format!(
@@ -365,16 +356,16 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
         T: ArrowPrimitiveType + ArrowDictionaryKeyType,
     {
         let mut builder: StringDictionaryBuilder<T> =
-            self.build_string_dictionary_builder(rows.len())?;
+            self.build_string_dictionary_builder(rows.len());
         for row in rows {
             if let Some(value) = self.field_lookup(col_name, row) {
-                if let Ok(str_v) = resolve_string(&value) {
+                if let Ok(str_v) = resolve_string(value) {
                     builder.append(str_v).map(drop)?
                 } else {
-                    builder.append_null()?
+                    builder.append_null()
                 }
             } else {
-                builder.append_null()?
+                builder.append_null()
             }
         }
         Ok(Arc::new(builder.finish()) as ArrayRef)
@@ -610,10 +601,8 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
             .iter()
             .filter(|field| projection.is_empty() || projection.contains(field.name()))
             .map(|field| {
-                match field.data_type() {
-                    DataType::Null => {
-                        Ok(Arc::new(NullArray::new(rows.len())) as ArrayRef)
-                    }
+                let arr = match field.data_type() {
+                    DataType::Null => Arc::new(NullArray::new(rows.len())) as ArrayRef,
                     DataType::Boolean => self.build_boolean_array(rows, field.name()),
                     DataType::Float64 => {
                         self.build_primitive_array::<Float64Type>(rows, field.name())
@@ -685,10 +674,12 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                                 rows,
                                 field.name(),
                             ),
-                        t => Err(ArrowError::SchemaError(format!(
-                            "TimeUnit {:?} not supported with Time64",
-                            t
-                        ))),
+                        t => {
+                            return Err(ArrowError::SchemaError(format!(
+                                "TimeUnit {:?} not supported with Time64",
+                                t
+                            )))
+                        }
                     },
                     DataType::Time32(unit) => match unit {
                         TimeUnit::Second => self
@@ -701,23 +692,23 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                                 rows,
                                 field.name(),
                             ),
-                        t => Err(ArrowError::SchemaError(format!(
-                            "TimeUnit {:?} not supported with Time32",
-                            t
-                        ))),
+                        t => {
+                            return Err(ArrowError::SchemaError(format!(
+                                "TimeUnit {:?} not supported with Time32",
+                                t
+                            )))
+                        }
                     },
-                    DataType::Utf8 | DataType::LargeUtf8 => Ok(Arc::new(
+                    DataType::Utf8 | DataType::LargeUtf8 => Arc::new(
                         rows.iter()
                             .map(|row| {
                                 let maybe_value = self.field_lookup(field.name(), row);
-                                maybe_value
-                                    .map(|value| resolve_string(&value))
-                                    .transpose()
+                                maybe_value.map(resolve_string).transpose()
                             })
                             .collect::<ArrowResult<StringArray>>()?,
                     )
-                        as ArrayRef),
-                    DataType::Binary | DataType::LargeBinary => Ok(Arc::new(
+                        as ArrayRef,
+                    DataType::Binary | DataType::LargeBinary => Arc::new(
                         rows.iter()
                             .map(|row| {
                                 let maybe_value = self.field_lookup(field.name(), row);
@@ -725,11 +716,11 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                             })
                             .collect::<BinaryArray>(),
                     )
-                        as ArrayRef),
+                        as ArrayRef,
                     DataType::List(ref list_field) => {
                         match list_field.data_type() {
                             DataType::Dictionary(ref key_ty, _) => {
-                                self.build_wrapped_list_array(rows, field.name(), key_ty)
+                                self.build_wrapped_list_array(rows, field.name(), key_ty)?
                             }
                             _ => {
                                 // extract rows by name
@@ -743,7 +734,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                                 self.build_nested_list_array::<i32>(
                                     extracted_rows.as_slice(),
                                     list_field,
-                                )
+                                )?
                             }
                         }
                     }
@@ -753,7 +744,7 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                             field.name(),
                             key_ty,
                             val_ty,
-                        ),
+                        )?,
                     DataType::Struct(fields) => {
                         let len = rows.len();
                         let num_bytes = bit_util::ceil(len, 8);
@@ -781,15 +772,17 @@ impl<'a, R: Read> AvroArrowArrayReader<'a, R> {
                             .child_data(
                                 arrays.into_iter().map(|a| a.data().clone()).collect(),
                             )
-                            .build()
-                            .unwrap();
-                        Ok(make_array(data))
+                            .build()?;
+                        make_array(data)
                     }
-                    _ => Err(ArrowError::SchemaError(format!(
-                        "type {:?} not supported",
-                        field.data_type()
-                    ))),
-                }
+                    _ => {
+                        return Err(ArrowError::SchemaError(format!(
+                            "type {:?} not supported",
+                            field.data_type()
+                        )))
+                    }
+                };
+                Ok(arr)
             })
             .collect();
         arrays
@@ -879,7 +872,7 @@ fn flatten_string_values(values: &[&Value]) -> Vec<Option<String>> {
 /// This is useful if the expected datatype is a string, in which case we preserve
 /// all the values regardless of they type.
 fn resolve_string(v: &Value) -> ArrowResult<String> {
-    let v = if let Value::Union(b) = v { b } else { v };
+    let v = if let Value::Union(_, b) = v { b } else { v };
     match v {
         Value::String(s) => Ok(s.clone()),
         Value::Bytes(bytes) => {
@@ -887,7 +880,7 @@ fn resolve_string(v: &Value) -> ArrowResult<String> {
         }
         other => Err(AvroError::GetString(other.into())),
     }
-    .map_err(|e| SchemaError(format!("expected resolvable string : {}", e)))
+    .map_err(|e| SchemaError(format!("expected resolvable string : {:?}", e)))
 }
 
 fn resolve_u8(v: &Value) -> AvroResult<u8> {
@@ -906,7 +899,7 @@ fn resolve_u8(v: &Value) -> AvroResult<u8> {
 }
 
 fn resolve_bytes(v: &Value) -> Option<Vec<u8>> {
-    let v = if let Value::Union(b) = v { b } else { v };
+    let v = if let Value::Union(_, b) = v { b } else { v };
     match v {
         Value::Bytes(_) => Ok(v.clone()),
         Value::String(s) => Ok(Value::Bytes(s.clone().into_bytes())),
@@ -927,7 +920,7 @@ fn resolve_bytes(v: &Value) -> Option<Vec<u8>> {
 }
 
 fn resolve_boolean(value: &Value) -> Option<bool> {
-    let v = if let Value::Union(b) = value {
+    let v = if let Value::Union(_, b) = value {
         b
     } else {
         value
@@ -950,7 +943,7 @@ fn maybe_resolve_union(value: &Value) -> &Value {
     if SchemaKind::from(value) == SchemaKind::Union {
         // Pull out the Union, and attempt to resolve against it.
         match value {
-            Value::Union(b) => b,
+            Value::Union(_, b) => b,
             _ => unreachable!(),
         }
     } else {
