@@ -162,7 +162,7 @@ pub fn split_files(
 pub async fn pruned_partition_list<'a>(
     store: &'a dyn ObjectStore,
     table_path: &'a ListingTableUrl,
-    filters: &[Expr],
+    filters: &'a [Expr],
     file_extension: &'a str,
     table_partition_cols: &'a [String],
 ) -> Result<BoxStream<'a, Result<PartitionedFile>>> {
@@ -183,24 +183,27 @@ pub async fn pruned_partition_list<'a>(
         // Note: We might avoid parsing the partition values if they are not used in any projection,
         // but the cost of parsing will likely be far dominated by the time to fetch the listing from
         // the object store.
-        Ok(Box::pin(list.try_filter_map(move |file_meta| async move {
-            let parsed_path = parse_partitions_for_path(
-                table_path,
-                &file_meta.location,
-                table_partition_cols,
-            )
-            .map(|p| {
-                p.iter()
-                    .map(|&pn| ScalarValue::Utf8(Some(pn.to_owned())))
-                    .collect()
-            });
+        Ok(Box::pin(list.try_filter_map(
+            move |object_meta| async move {
+                let parsed_path = parse_partitions_for_path(
+                    table_path,
+                    &object_meta.location,
+                    table_partition_cols,
+                )
+                .map(|p| {
+                    p.iter()
+                        .map(|&pn| ScalarValue::Utf8(Some(pn.to_owned())))
+                        .collect()
+                });
 
-            Ok(parsed_path.map(|partition_values| PartitionedFile {
-                partition_values,
-                object_meta: file_meta,
-                range: None,
-            }))
-        })))
+                Ok(parsed_path.map(|partition_values| PartitionedFile {
+                    partition_values,
+                    object_meta,
+                    range: None,
+                    extensions: None,
+                }))
+            },
+        )))
     } else {
         // parse the partition values and serde them as a RecordBatch to filter them
         let metas: Vec<_> = list.try_collect().await?;
@@ -247,11 +250,11 @@ fn paths_to_batch(
             &file_meta.location,
             table_partition_cols,
         ) {
-            key_builder.append_value(file_meta.location.as_ref())?;
-            length_builder.append_value(file_meta.size as u64)?;
-            modified_builder.append_value(file_meta.last_modified.timestamp_millis())?;
+            key_builder.append_value(file_meta.location.as_ref());
+            length_builder.append_value(file_meta.size as u64);
+            modified_builder.append_value(file_meta.last_modified.timestamp_millis());
             for (i, part_val) in partition_values.iter().enumerate() {
-                partition_builders[i].append_value(part_val)?;
+                partition_builders[i].append_value(part_val);
             }
         } else {
             debug!("No partitioning for path {}", file_meta.location);
@@ -317,6 +320,7 @@ fn batches_to_paths(batches: &[RecordBatch]) -> Result<Vec<PartitionedFile>> {
                         })
                         .collect(),
                     range: None,
+                    extensions: None,
                 })
             })
         })
