@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 #![warn(missing_docs, clippy::needless_borrow)]
+// TODO: Temporary workaround for https://github.com/apache/arrow-rs/issues/2372 (#3081)
+#![allow(where_clauses_object_safety)]
 
 //! [DataFusion](https://github.com/apache/arrow-datafusion)
 //! is an extensible query execution framework that uses
@@ -237,7 +239,7 @@ pub use datafusion_sql as sql;
 #[cfg(feature = "jit")]
 pub use datafusion_jit as jit;
 
-pub use physical_expr::from_slice;
+pub use common::from_slice;
 
 #[cfg(test)]
 pub mod test;
