@@ -28,18 +28,18 @@ pub use datafusion_common::{
 };
 pub use datafusion_expr::{
     abs, acos, and, approx_distinct, approx_percentile_cont, array, ascii, asin, atan,
-    avg, bit_length, btrim, call_fn, case, ceil, character_length, chr, coalesce, col,
-    combine_filters, concat, concat_expr, concat_ws, concat_ws_expr, cos, count,
-    count_distinct, create_udaf, create_udf, date_part, date_trunc, digest, exists, exp,
-    expr_rewriter,
+    atan2, avg, bit_length, btrim, call_fn, case, cast, ceil, character_length, chr,
+    coalesce, col, combine_filters, concat, concat_expr, concat_ws, concat_ws_expr, cos,
+    count, count_distinct, create_udaf, create_udf, date_part, date_trunc, digest,
+    exists, exp, expr_rewriter,
     expr_rewriter::{
         normalize_col, normalize_col_with_schemas, normalize_cols, replace_col,
         rewrite_sort_cols_by_aggs, unnormalize_col, unnormalize_cols, ExprRewritable,
         ExprRewriter, RewriteRecursion,
     },
     expr_visitor::{ExprVisitable, ExpressionVisitor, Recursion},
-    floor, in_list, in_subquery, initcap, left, length, lit, lit_timestamp_nano, ln,
-    log10, log2,
+    floor, from_unixtime, in_list, in_subquery, initcap, left, length, lit,
+    lit_timestamp_nano, ln, log10, log2,
     logical_plan::{
         builder::{
             build_join_schema, union_with_alias, LogicalPlanBuilder, UNNAMED_TABLE,
