@@ -25,37 +25,42 @@ use std::collections::VecDeque;
 use std::pin::Pin;
 use std::sync::Arc;
 use std::task::{Context, Poll};
+use std::time::Instant;
 
 use arrow::datatypes::SchemaRef;
 use arrow::{error::Result as ArrowResult, record_batch::RecordBatch};
 use futures::future::BoxFuture;
 use futures::stream::BoxStream;
 use futures::{ready, FutureExt, Stream, StreamExt};
-use object_store::{ObjectMeta, ObjectStore};
+use object_store::ObjectStore;
 
 use datafusion_common::ScalarValue;
 
-use crate::datasource::listing::{FileRange, PartitionedFile};
+use crate::datasource::listing::PartitionedFile;
 use crate::error::Result;
 use crate::execution::context::TaskContext;
-use crate::physical_plan::file_format::{FileScanConfig, PartitionColumnProjector};
+use crate::physical_plan::file_format::{
+    FileMeta, FileScanConfig, PartitionColumnProjector,
+};
+use crate::physical_plan::metrics::{
+    BaselineMetrics, ExecutionPlanMetricsSet, MetricBuilder, Time,
+};
 use crate::physical_plan::RecordBatchStream;
 
 /// A fallible future that resolves to a stream of [`RecordBatch`]
-pub type ReaderFuture =
+pub type FileOpenFuture =
     BoxFuture<'static, Result<BoxStream<'static, ArrowResult<RecordBatch>>>>;
 
-pub trait FormatReader: Unpin {
+pub trait FileOpener: Unpin {
     fn open(
         &self,
         store: Arc<dyn ObjectStore>,
-        file: ObjectMeta,
-        range: Option<FileRange>,
-    ) -> ReaderFuture;
+        file_meta: FileMeta,
+    ) -> Result<FileOpenFuture>;
 }
 
 /// A stream that iterates record batch by record batch, file over file.
-pub struct FileStream<F: FormatReader> {
+pub struct FileStream<F: FileOpener> {
     /// An iterator over input files.
     file_iter: VecDeque<PartitionedFile>,
     /// The stream schema (file schema including partition columns and after
@@ -74,6 +79,10 @@ pub struct FileStream<F: FormatReader> {
     object_store: Arc<dyn ObjectStore>,
     /// The stream state
     state: FileStreamState,
+    /// File stream specific metrics
+    file_stream_metrics: FileStreamMetrics,
+    /// runtime baseline metrics
+    baseline_metrics: BaselineMetrics,
 }
 
 enum FileStreamState {
@@ -82,12 +91,12 @@ enum FileStreamState {
     /// Currently performing asynchronous IO to obtain a stream of RecordBatch
     /// for a given parquet file
     Open {
-        /// A [`ReaderFuture`] returned by [`FormatReader::open`]
-        future: ReaderFuture,
+        /// A [`FileOpenFuture`] returned by [`FormatReader::open`]
+        future: FileOpenFuture,
         /// The partition values for this file
         partition_values: Vec<ScalarValue>,
     },
-    /// Scanning the [`BoxStream`] returned by the completion of a [`ReaderFuture`]
+    /// Scanning the [`BoxStream`] returned by the completion of a [`FileOpenFuture`]
     /// returned by [`FormatReader::open`]
     Scan {
         /// Partitioning column values for the current batch_iter
@@ -101,12 +110,69 @@ enum FileStreamState {
     Limit,
 }
 
-impl<F: FormatReader> FileStream<F> {
+struct StartableTime {
+    metrics: Time,
+    // use for record each part cost time, will eventually add into 'metrics'.
+    start: Option<Instant>,
+}
+
+impl StartableTime {
+    fn start(&mut self) {
+        assert!(self.start.is_none());
+        self.start = Some(Instant::now());
+    }
+
+    fn stop(&mut self) {
+        if let Some(start) = self.start.take() {
+            self.metrics.add_elapsed(start);
+        }
+    }
+}
+
+struct FileStreamMetrics {
+    /// Time elapsed for file opening
+    pub time_opening: StartableTime,
+    /// Time elapsed for file scanning + first record batch of decompression + decoding
+    pub time_scanning: StartableTime,
+    /// Time elapsed for data decompression + decoding
+    pub time_processing: StartableTime,
+}
+
+impl FileStreamMetrics {
+    fn new(metrics: &ExecutionPlanMetricsSet, partition: usize) -> Self {
+        let time_opening = StartableTime {
+            metrics: MetricBuilder::new(metrics)
+                .subset_time("time_elapsed_opening", partition),
+            start: None,
+        };
+
+        let time_scanning = StartableTime {
+            metrics: MetricBuilder::new(metrics)
+                .subset_time("time_elapsed_scanning", partition),
+            start: None,
+        };
+
+        let time_processing = StartableTime {
+            metrics: MetricBuilder::new(metrics)
+                .subset_time("time_elapsed_processing", partition),
+            start: None,
+        };
+
+        Self {
+            time_opening,
+            time_scanning,
+            time_processing,
+        }
+    }
+}
+
+impl<F: FileOpener> FileStream<F> {
     pub fn new(
         config: &FileScanConfig,
         partition: usize,
         context: Arc<TaskContext>,
         file_reader: F,
+        metrics: ExecutionPlanMetricsSet,
     ) -> Result<Self> {
         let (projected_schema, _) = config.project();
         let pc_projector = PartitionColumnProjector::new(
@@ -128,6 +194,8 @@ impl<F: FormatReader> FileStream<F> {
             pc_projector,
             object_store,
             state: FileStreamState::Idle,
+            file_stream_metrics: FileStreamMetrics::new(&metrics, partition),
+            baseline_metrics: BaselineMetrics::new(&metrics, partition),
         })
     }
 
@@ -138,20 +206,30 @@ impl<F: FormatReader> FileStream<F> {
         loop {
             match &mut self.state {
                 FileStreamState::Idle => {
-                    let file = match self.file_iter.pop_front() {
+                    let part_file = match self.file_iter.pop_front() {
                         Some(file) => file,
                         None => return Poll::Ready(None),
                     };
 
-                    let future = self.file_reader.open(
-                        self.object_store.clone(),
-                        file.object_meta,
-                        file.range,
-                    );
+                    let file_meta = FileMeta {
+                        object_meta: part_file.object_meta,
+                        range: part_file.range,
+                        extensions: part_file.extensions,
+                    };
 
-                    self.state = FileStreamState::Open {
-                        future,
-                        partition_values: file.partition_values,
+                    self.file_stream_metrics.time_opening.start();
+
+                    match self.file_reader.open(self.object_store.clone(), file_meta) {
+                        Ok(future) => {
+                            self.state = FileStreamState::Open {
+                                future,
+                                partition_values: part_file.partition_values,
+                            }
+                        }
+                        Err(e) => {
+                            self.state = FileStreamState::Error;
+                            return Poll::Ready(Some(Err(e.into())));
+                        }
                     }
                 }
                 FileStreamState::Open {
@@ -159,6 +237,8 @@ impl<F: FormatReader> FileStream<F> {
                     partition_values,
                 } => match ready!(future.poll_unpin(cx)) {
                     Ok(reader) => {
+                        self.file_stream_metrics.time_opening.stop();
+                        self.file_stream_metrics.time_scanning.start();
                         self.state = FileStreamState::Scan {
                             partition_values: std::mem::take(partition_values),
                             reader,
@@ -174,6 +254,7 @@ impl<F: FormatReader> FileStream<F> {
                     partition_values,
                 } => match ready!(reader.poll_next_unpin(cx)) {
                     Some(result) => {
+                        self.file_stream_metrics.time_scanning.stop();
                         let result = result
                             .and_then(|b| self.pc_projector.project(b, partition_values))
                             .map(|batch| match &mut self.remain {
@@ -197,7 +278,10 @@ impl<F: FormatReader> FileStream<F> {
 
                         return Poll::Ready(Some(result));
                     }
-                    None => self.state = FileStreamState::Idle,
+                    None => {
+                        self.file_stream_metrics.time_scanning.stop();
+                        self.state = FileStreamState::Idle;
+                    }
                 },
                 FileStreamState::Error | FileStreamState::Limit => {
                     return Poll::Ready(None)
@@ -207,18 +291,21 @@ impl<F: FormatReader> FileStream<F> {
     }
 }
 
-impl<F: FormatReader> Stream for FileStream<F> {
+impl<F: FileOpener> Stream for FileStream<F> {
     type Item = ArrowResult<RecordBatch>;
 
     fn poll_next(
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
     ) -> Poll<Option<Self::Item>> {
-        self.poll_inner(cx)
+        self.file_stream_metrics.time_processing.start();
+        let result = self.poll_inner(cx);
+        self.file_stream_metrics.time_processing.stop();
+        self.baseline_metrics.record_poll(result)
     }
 }
 
-impl<F: FormatReader> RecordBatchStream for FileStream<F> {
+impl<F: FileOpener> RecordBatchStream for FileStream<F> {
     fn schema(&self) -> SchemaRef {
         self.projected_schema.clone()
     }
@@ -230,6 +317,7 @@ mod tests {
 
     use super::*;
     use crate::datasource::object_store::ObjectStoreUrl;
+    use crate::physical_plan::metrics::ExecutionPlanMetricsSet;
     use crate::prelude::SessionContext;
     use crate::{
         error::Result,
@@ -240,16 +328,15 @@ mod tests {
         records: Vec<RecordBatch>,
     }
 
-    impl FormatReader for TestOpener {
+    impl FileOpener for TestOpener {
         fn open(
             &self,
             _store: Arc<dyn ObjectStore>,
-            _file: ObjectMeta,
-            _range: Option<FileRange>,
-        ) -> ReaderFuture {
+            _file_meta: FileMeta,
+        ) -> Result<FileOpenFuture> {
             let iterator = self.records.clone().into_iter().map(Ok);
             let stream = futures::stream::iter(iterator).boxed();
-            futures::future::ready(Ok(stream)).boxed()
+            Ok(futures::future::ready(Ok(stream)).boxed())
         }
     }
 
@@ -276,7 +363,14 @@ mod tests {
             table_partition_cols: vec![],
         };
 
-        let file_stream = FileStream::new(&config, 0, ctx.task_ctx(), reader).unwrap();
+        let file_stream = FileStream::new(
+            &config,
+            0,
+            ctx.task_ctx(),
+            reader,
+            ExecutionPlanMetricsSet::new(),
+        )
+        .unwrap();
 
         file_stream
             .map(|b| b.expect("No error expected in stream"))
