@@ -16,23 +16,24 @@
 // under the License.
 
 //! Execution plan for reading line-delimited JSON files
-use arrow::json::reader::DecoderOptions;
-
-use crate::datasource::listing::FileRange;
 use crate::error::{DataFusionError, Result};
 use crate::execution::context::SessionState;
 use crate::execution::context::TaskContext;
 use crate::physical_plan::expressions::PhysicalSortExpr;
+use crate::physical_plan::file_format::delimited_stream::newline_delimited_stream;
 use crate::physical_plan::file_format::file_stream::{
-    FileStream, FormatReader, ReaderFuture,
+    FileOpenFuture, FileOpener, FileStream,
 };
+use crate::physical_plan::file_format::FileMeta;
+use crate::physical_plan::metrics::ExecutionPlanMetricsSet;
 use crate::physical_plan::{
     DisplayFormatType, ExecutionPlan, Partitioning, SendableRecordBatchStream, Statistics,
 };
+use arrow::json::reader::DecoderOptions;
 use arrow::{datatypes::SchemaRef, json};
 use bytes::Buf;
 use futures::{StreamExt, TryStreamExt};
-use object_store::{GetResult, ObjectMeta, ObjectStore};
+use object_store::{GetResult, ObjectStore};
 use std::any::Any;
 use std::fs;
 use std::path::Path;
@@ -47,6 +48,8 @@ pub struct NdJsonExec {
     base_config: FileScanConfig,
     projected_statistics: Statistics,
     projected_schema: SchemaRef,
+    /// Execution metrics
+    metrics: ExecutionPlanMetricsSet,
 }
 
 impl NdJsonExec {
@@ -58,6 +61,7 @@ impl NdJsonExec {
             base_config,
             projected_schema,
             projected_statistics,
+            metrics: ExecutionPlanMetricsSet::new(),
         }
     }
 }
@@ -116,7 +120,13 @@ impl ExecutionPlan for NdJsonExec {
             options,
         };
 
-        let stream = FileStream::new(&self.base_config, partition, context, opener)?;
+        let stream = FileStream::new(
+            &self.base_config,
+            partition,
+            context,
+            opener,
+            self.metrics.clone(),
+        )?;
 
         Ok(Box::pin(stream) as SendableRecordBatchStream)
     }
@@ -148,30 +158,35 @@ struct JsonOpener {
     file_schema: SchemaRef,
 }
 
-impl FormatReader for JsonOpener {
+impl FileOpener for JsonOpener {
     fn open(
         &self,
         store: Arc<dyn ObjectStore>,
-        file: ObjectMeta,
-        _range: Option<FileRange>,
-    ) -> ReaderFuture {
+        file_meta: FileMeta,
+    ) -> Result<FileOpenFuture> {
         let options = self.options.clone();
         let schema = self.file_schema.clone();
-        Box::pin(async move {
-            match store.get(&file.location).await? {
+        Ok(Box::pin(async move {
+            match store.get(file_meta.location()).await? {
                 GetResult::File(file, _) => {
                     let reader = json::Reader::new(file, schema.clone(), options);
                     Ok(futures::stream::iter(reader).boxed())
                 }
-                r @ GetResult::Stream(_) => {
-                    let bytes = r.bytes().await?;
-                    let reader =
-                        json::Reader::new(bytes.reader(), schema.clone(), options);
-
-                    Ok(futures::stream::iter(reader).boxed())
+                GetResult::Stream(s) => {
+                    Ok(newline_delimited_stream(s.map_err(Into::into))
+                        .map_ok(move |bytes| {
+                            let reader = json::Reader::new(
+                                bytes.reader(),
+                                schema.clone(),
+                                options.clone(),
+                            );
+                            futures::stream::iter(reader)
+                        })
+                        .try_flatten()
+                        .boxed())
                 }
             }
-        })
+        }))
     }
 }
 
@@ -218,10 +233,13 @@ mod tests {
     use arrow::array::Array;
     use arrow::datatypes::{Field, Schema};
     use futures::StreamExt;
+    use object_store::local::LocalFileSystem;
 
+    use crate::assert_batches_eq;
     use crate::datasource::file_format::{json::JsonFormat, FileFormat};
     use crate::datasource::listing::PartitionedFile;
     use crate::datasource::object_store::ObjectStoreUrl;
+    use crate::physical_plan::file_format::chunked_store::ChunkedStore;
     use crate::prelude::NdJsonReadOptions;
     use crate::prelude::*;
     use crate::test::object_store::local_unpartitioned_file;
@@ -433,4 +451,38 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_chunked() {
+        let mut ctx = SessionContext::new();
+
+        for chunk_size in [10, 20, 30, 40] {
+            ctx.runtime_env().register_object_store(
+                "file",
+                "",
+                Arc::new(ChunkedStore::new(
+                    Arc::new(LocalFileSystem::new()),
+                    chunk_size,
+                )),
+            );
+
+            let path = format!("{}/1.json", TEST_DATA_BASE);
+            let frame = ctx.read_json(path, Default::default()).await.unwrap();
+            let results = frame.collect().await.unwrap();
+
+            assert_batches_eq!(
+                &[
+                    "+-----+----------------+---------------+------+",
+                    "| a   | b              | c             | d    |",
+                    "+-----+----------------+---------------+------+",
+                    "| 1   | [2, 1.3, -6.1] | [false, true] | 4    |",
+                    "| -10 | [2, 1.3, -6.1] | [true, true]  | 4    |",
+                    "| 2   | [2, , -6.1]    | [false, ]     | text |",
+                    "|     |                |               |      |",
+                    "+-----+----------------+---------------+------+",
+                ],
+                &results
+            );
+        }
+    }
 }
