@@ -21,7 +21,7 @@ use crate::error::{DataFusionError, Result};
 use ahash::{CallHasher, RandomState};
 use arrow::array::{
     Array, ArrayRef, BasicDecimalArray, BooleanArray, Date32Array, Date64Array,
-    DecimalArray, DictionaryArray, Float32Array, Float64Array, Int16Array, Int32Array,
+    Decimal128Array, DictionaryArray, Float32Array, Float64Array, Int16Array, Int32Array,
     Int64Array, Int8Array, LargeStringArray, StringArray, TimestampMicrosecondArray,
     TimestampMillisecondArray, TimestampNanosecondArray, TimestampSecondArray,
     UInt16Array, UInt32Array, UInt64Array, UInt8Array,
@@ -58,7 +58,7 @@ fn hash_decimal128<'a>(
     hashes_buffer: &'a mut [u64],
     mul_col: bool,
 ) {
-    let array = array.as_any().downcast_ref::<DecimalArray>().unwrap();
+    let array = array.as_any().downcast_ref::<Decimal128Array>().unwrap();
     if array.null_count() == 0 {
         if mul_col {
             for (i, hash) in hashes_buffer.iter_mut().enumerate() {
@@ -91,19 +91,19 @@ fn hash_decimal128<'a>(
 }
 
 macro_rules! hash_array {
-    ($array_type:ident, $column: ident, $ty: ident, $hashes: ident, $random_state: ident, $multi_col: ident) => {
+    ($array_type:ident, $column: ident, $ty: ty, $hashes: ident, $random_state: ident, $multi_col: ident) => {
         let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
         if array.null_count() == 0 {
             if $multi_col {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
                     *hash = combine_hashes(
-                        $ty::get_hash(&array.value(i), $random_state),
+                        <$ty>::get_hash(&array.value(i), $random_state),
                         *hash,
                     );
                 }
             } else {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
-                    *hash = $ty::get_hash(&array.value(i), $random_state);
+                    *hash = <$ty>::get_hash(&array.value(i), $random_state);
                 }
             }
         } else {
@@ -111,7 +111,7 @@ macro_rules! hash_array {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
                     if !array.is_null(i) {
                         *hash = combine_hashes(
-                            $ty::get_hash(&array.value(i), $random_state),
+                            <$ty>::get_hash(&array.value(i), $random_state),
                             *hash,
                         );
                     }
@@ -119,7 +119,7 @@ macro_rules! hash_array {
             } else {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
                     if !array.is_null(i) {
-                        *hash = $ty::get_hash(&array.value(i), $random_state);
+                        *hash = <$ty>::get_hash(&array.value(i), $random_state);
                     }
                 }
             }
@@ -329,6 +329,8 @@ pub fn create_hashes<'a>(
     hashes_buffer: &'a mut Vec<u64>,
 ) -> Result<&'a mut Vec<u64>> {
     // combine hashes with `combine_hashes` if we have more than 1 column
+
+    use arrow::array::{BinaryArray, LargeBinaryArray};
     let multi_col = arrays.len() > 1;
 
     for col in arrays {
@@ -336,7 +338,7 @@ pub fn create_hashes<'a>(
             DataType::Null => {
                 hash_null(random_state, hashes_buffer, multi_col);
             }
-            DataType::Decimal(_, _) => {
+            DataType::Decimal128(_, _) => {
                 hash_decimal128(col, random_state, hashes_buffer, multi_col);
             }
             DataType::UInt8 => {
@@ -529,6 +531,26 @@ pub fn create_hashes<'a>(
                     multi_col
                 );
             }
+            DataType::Binary => {
+                hash_array!(
+                    BinaryArray,
+                    col,
+                    &[u8],
+                    hashes_buffer,
+                    random_state,
+                    multi_col
+                );
+            }
+            DataType::LargeBinary => {
+                hash_array!(
+                    LargeBinaryArray,
+                    col,
+                    &[u8],
+                    hashes_buffer,
+                    random_state,
+                    multi_col
+                );
+            }
             DataType::Dictionary(index_type, _) => match **index_type {
                 DataType::Int8 => {
                     create_hashes_dictionary::<Int8Type>(
@@ -616,7 +638,10 @@ pub fn create_hashes<'a>(
 #[cfg(test)]
 mod tests {
     use crate::from_slice::FromSlice;
-    use arrow::{array::DictionaryArray, datatypes::Int8Type};
+    use arrow::{
+        array::{BinaryArray, DictionaryArray},
+        datatypes::Int8Type,
+    };
     use std::sync::Arc;
 
     use super::*;
@@ -626,7 +651,7 @@ mod tests {
         let array = vec![1, 2, 3, 4]
             .into_iter()
             .map(Some)
-            .collect::<DecimalArray>()
+            .collect::<Decimal128Array>()
             .with_precision_and_scale(20, 3)
             .unwrap();
         let array_ref = Arc::new(array);
@@ -653,6 +678,22 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn create_hashes_binary() -> Result<()> {
+        let byte_array = Arc::new(BinaryArray::from_vec(vec![
+            &[4, 3, 2],
+            &[4, 3, 2],
+            &[1, 2, 3],
+        ]));
+
+        let random_state = RandomState::with_seeds(0, 0, 0, 0);
+        let hashes_buff = &mut vec![0; byte_array.len()];
+        let hashes = create_hashes(&[byte_array], &random_state, hashes_buff)?;
+        assert_eq!(hashes.len(), 3,);
+
+        Ok(())
+    }
+
     #[test]
     // Tests actual values of hashes, which are different if forcing collisions
     #[cfg(not(feature = "force_hash_collisions"))]
