@@ -134,7 +134,7 @@ impl ExternalSorter {
     /// MergeSort in mem batches as well as spills into total order with `SortPreservingMergeStream`.
     async fn sort(&self) -> Result<SendableRecordBatchStream> {
         let partition = self.partition_id();
-        let batch_size = self.session_config.batch_size;
+        let batch_size = self.session_config.batch_size();
         let mut in_mem_batches = self.in_mem_batches.lock().await;
 
         if self.spilled_before().await {
@@ -144,7 +144,7 @@ impl ExternalSorter {
             let mut streams: Vec<SortedStream> = vec![];
             if in_mem_batches.len() > 0 {
                 let in_mem_stream = in_mem_partial_sort(
-                    &mut *in_mem_batches,
+                    &mut in_mem_batches,
                     self.schema.clone(),
                     &self.expr,
                     batch_size,
@@ -168,14 +168,14 @@ impl ExternalSorter {
                 self.schema.clone(),
                 &self.expr,
                 tracking_metrics,
-                self.session_config.batch_size,
+                self.session_config.batch_size(),
             )))
         } else if in_mem_batches.len() > 0 {
             let tracking_metrics = self
                 .metrics_set
                 .new_final_tracking(partition, self.runtime.clone());
             let result = in_mem_partial_sort(
-                &mut *in_mem_batches,
+                &mut in_mem_batches,
                 self.schema.clone(),
                 &self.expr,
                 batch_size,
@@ -265,10 +265,10 @@ impl MemoryConsumer for ExternalSorter {
 
         let spillfile = self.runtime.disk_manager.create_tmp_file()?;
         let stream = in_mem_partial_sort(
-            &mut *in_mem_batches,
+            &mut in_mem_batches,
             self.schema.clone(),
-            &*self.expr,
-            self.session_config.batch_size,
+            &self.expr,
+            self.session_config.batch_size(),
             tracking_metrics,
         );
 
