@@ -239,7 +239,7 @@ async fn custom_source_dataframe() -> Result<()> {
 
     let expected = format!(
         "Projection: #{}.c2\
-        \n  TableScan: {} projection=Some([c2])",
+        \n  TableScan: {} projection=[c2]",
         UNNAMED_TABLE, UNNAMED_TABLE
     );
     assert_eq!(format!("{:?}", optimized_plan), expected);
