@@ -33,6 +33,7 @@ use datafusion::prelude::*;
 use datafusion::execution::context::SessionContext;
 
 use datafusion::assert_batches_eq;
+use datafusion_expr::{approx_median, cast};
 
 fn create_test_table() -> Result<Arc<DataFrame>> {
     let schema = Arc::new(Schema::new(vec![
@@ -152,6 +153,26 @@ async fn test_fn_btrim_with_chars() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn test_fn_approx_median() -> Result<()> {
+    let expr = approx_median(col("b"));
+
+    let expected = vec![
+        "+----------------------+",
+        "| APPROXMEDIAN(test.b) |",
+        "+----------------------+",
+        "| 10                   |",
+        "+----------------------+",
+    ];
+
+    let df = create_test_table()?;
+    let batches = df.aggregate(vec![], vec![expr]).unwrap().collect().await?;
+
+    assert_batches_eq!(expected, &batches);
+
+    Ok(())
+}
+
 #[tokio::test]
 async fn test_fn_approx_percentile_cont() -> Result<()> {
     let expr = approx_percentile_cont(col("b"), lit(0.5));
@@ -642,6 +663,25 @@ async fn test_fn_substr() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn test_cast() -> Result<()> {
+    let expr = cast(col("b"), DataType::Float64);
+    let expected = vec![
+        "+-------------------------+",
+        "| CAST(test.b AS Float64) |",
+        "+-------------------------+",
+        "| 1                       |",
+        "| 10                      |",
+        "| 10                      |",
+        "| 100                     |",
+        "+-------------------------+",
+    ];
+
+    assert_fn_batches!(expr, expected);
+
+    Ok(())
+}
+
 #[tokio::test]
 async fn test_fn_to_hex() -> Result<()> {
     let expr = to_hex(col("b"));
