@@ -36,7 +36,7 @@ use std::sync::Arc;
 fn create_batch(value: i32, num_rows: usize) -> Result<RecordBatch> {
     let mut builder = Int32Builder::new(num_rows);
     for _ in 0..num_rows {
-        builder.append_value(value)?;
+        builder.append_value(value);
     }
 
     Ok(RecordBatch::try_new(
