@@ -505,6 +505,9 @@ async fn test_mathematical_expressions_with_null() -> Result<()> {
     test_expression!("power(NULL, 2)", "NULL");
     test_expression!("power(NULL, NULL)", "NULL");
     test_expression!("power(2, NULL)", "NULL");
+    test_expression!("atan2(NULL, NULL)", "NULL");
+    test_expression!("atan2(1, NULL)", "NULL");
+    test_expression!("atan2(NULL, 1)", "NULL");
     Ok(())
 }
 
@@ -641,6 +644,16 @@ async fn test_struct_literals() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn binary_bitwise_shift() -> Result<()> {
+    test_expression!("2 << 10", "2048");
+    test_expression!("2048 >> 10", "2");
+    test_expression!("2048 << NULL", "NULL");
+    test_expression!("2048 >> NULL", "NULL");
+
+    Ok(())
+}
+
 #[tokio::test]
 async fn test_interval_expressions() -> Result<()> {
     // day nano intervals
@@ -688,6 +701,14 @@ async fn test_interval_expressions() -> Result<()> {
         "interval '1 day'",
         "0 years 0 mons 1 days 0 hours 0 mins 0.00 secs"
     );
+    test_expression!(
+        "interval '1 week'",
+        "0 years 0 mons 7 days 0 hours 0 mins 0.00 secs"
+    );
+    test_expression!(
+        "interval '2 weeks'",
+        "0 years 0 mons 14 days 0 hours 0 mins 0.00 secs"
+    );
     test_expression!(
         "interval '1 day 1'",
         "0 years 0 mons 1 days 0 hours 0 mins 1.00 secs"
@@ -766,6 +787,18 @@ async fn test_interval_expressions() -> Result<()> {
         "interval '1 year'",
         "1 years 0 mons 0 days 0 hours 0 mins 0.00 secs"
     );
+    test_expression!(
+        "interval '1 decade'",
+        "10 years 0 mons 0 days 0 hours 0 mins 0.00 secs"
+    );
+    test_expression!(
+        "interval '2 decades'",
+        "20 years 0 mons 0 days 0 hours 0 mins 0.00 secs"
+    );
+    test_expression!(
+        "interval '1 century'",
+        "100 years 0 mons 0 days 0 hours 0 mins 0.00 secs"
+    );
     test_expression!(
         "interval '2 year'",
         "2 years 0 mons 0 days 0 hours 0 mins 0.00 secs"
