@@ -111,7 +111,6 @@ async fn query_concat() -> Result<()> {
     Ok(())
 }
 
-// Revisit after implementing https://github.com/apache/arrow-rs/issues/925
 #[tokio::test]
 async fn query_array() -> Result<()> {
     let schema = Arc::new(Schema::new(vec![
@@ -132,14 +131,18 @@ async fn query_array() -> Result<()> {
     let ctx = SessionContext::new();
     ctx.register_table("test", Arc::new(table))?;
     let sql = "SELECT array(c1, cast(c2 as varchar)) FROM test";
-    let actual = execute(&ctx, sql).await;
+    let actual = execute_to_batches(&ctx, sql).await;
     let expected = vec![
-        vec!["[,0]"],
-        vec!["[a,1]"],
-        vec!["[aa,NULL]"],
-        vec!["[aaa,3]"],
+        "+--------------------------------------+",
+        "| array(test.c1,CAST(test.c2 AS Utf8)) |",
+        "+--------------------------------------+",
+        "| [, 0]                                |",
+        "| [a, 1]                               |",
+        "| [aa, ]                               |",
+        "| [aaa, 3]                             |",
+        "+--------------------------------------+",
     ];
-    assert_eq!(expected, actual);
+    assert_batches_eq!(expected, &actual);
     Ok(())
 }
 
@@ -148,9 +151,15 @@ async fn query_array_scalar() -> Result<()> {
     let ctx = SessionContext::new();
 
     let sql = "SELECT array(1, 2, 3);";
-    let actual = execute(&ctx, sql).await;
-    let expected = vec![vec!["[1, 2, 3]"]];
-    assert_eq!(expected, actual);
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+-----------------------------------+",
+        "| array(Int64(1),Int64(2),Int64(3)) |",
+        "+-----------------------------------+",
+        "| [1, 2, 3]                         |",
+        "+-----------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
     Ok(())
 }
 
@@ -495,9 +504,9 @@ async fn test_power() -> Result<()> {
     ctx.register_table("test", Arc::new(table))?;
     let sql = r"SELECT power(i32, exp_i) as power_i32,
                  power(i64, exp_f) as power_i64,
-                 power(f32, exp_i) as power_f32,
+                 pow(f32, exp_i) as power_f32,
                  power(f64, exp_f) as power_f64,
-                 power(2, 3) as power_int_scalar,
+                 pow(2, 3) as power_int_scalar,
                  power(2.5, 3.0) as power_float_scalar
           FROM (select test.*, 3 as exp_i, 3.0 as exp_f from test) a";
     let actual = execute_to_batches(&ctx, sql).await;
