@@ -17,6 +17,7 @@
 
 use super::*;
 use datafusion::from_slice::FromSlice;
+use std::ops::Add;
 
 #[tokio::test]
 async fn query_cast_timestamp_millis() -> Result<()> {
@@ -266,6 +267,37 @@ async fn query_cast_timestamp_micros_to_others() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn query_cast_timestamp_from_unixtime() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let t1_schema = Arc::new(Schema::new(vec![Field::new("ts", DataType::Int64, true)]));
+    let t1_data = RecordBatch::try_new(
+        t1_schema.clone(),
+        vec![Arc::new(Int64Array::from(vec![
+            1235865600, 1235865660, 1238544000,
+        ]))],
+    )?;
+    let t1_table = MemTable::try_new(t1_schema, vec![vec![t1_data]])?;
+    ctx.register_table("t1", Arc::new(t1_table))?;
+
+    let sql = "SELECT from_unixtime(ts) FROM t1 LIMIT 3";
+    let actual = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+---------------------+",
+        "| fromunixtime(t1.ts) |",
+        "+---------------------+",
+        "| 2009-03-01 00:00:00 |",
+        "| 2009-03-01 00:01:00 |",
+        "| 2009-04-01 00:00:00 |",
+        "+---------------------+",
+    ];
+
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
 #[tokio::test]
 async fn to_timestamp() -> Result<()> {
     let ctx = SessionContext::new();
@@ -347,6 +379,25 @@ async fn to_timestamp_seconds() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn from_unixtime() -> Result<()> {
+    let ctx = SessionContext::new();
+    ctx.register_table("ts_data", make_timestamp_table::<TimestampSecondType>()?)?;
+
+    let sql = "SELECT COUNT(*) FROM ts_data where ts > from_unixtime(1599566400)"; // '2020-09-08T12:00:00+00:00'
+    let actual = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+-----------------+",
+        "| COUNT(UInt8(1)) |",
+        "+-----------------+",
+        "| 2               |",
+        "+-----------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
 #[tokio::test]
 async fn count_distinct_timestamps() -> Result<()> {
     let ctx = SessionContext::new();
@@ -814,3 +865,603 @@ async fn group_by_timestamp_millis() -> Result<()> {
     assert_batches_eq!(expected, &actual);
     Ok(())
 }
+
+#[tokio::test]
+async fn interval_year() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-01-01' + interval '1' year as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1995-01-01 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn add_interval_month() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-01-31' + interval '1' month as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1994-02-28 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn sub_interval_month() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-03-31' - interval '1' month as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1994-02-28 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn sub_month_wrap() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-01-15' - interval '1' month as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1993-12-15 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn add_interval_day() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-01-15' + interval '1' day as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1994-01-16 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn sub_interval_day() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select date '1994-01-01' - interval '1' day as date;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+------------+",
+        "| date       |",
+        "+------------+",
+        "| 1993-12-31 |",
+        "+------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn cast_string_to_time() {
+    let ctx = SessionContext::new();
+
+    let sql = "select \
+        time '08:09:10.123456789' as time_nano, \
+        time '13:14:15.123456'    as time_micro,\
+        time '13:14:15.123'       as time_milli,\
+        time '13:14:15'           as time;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+--------------------+-----------------+--------------+----------+",
+        "| time_nano          | time_micro      | time_milli   | time     |",
+        "+--------------------+-----------------+--------------+----------+",
+        "| 08:09:10.123456789 | 13:14:15.123456 | 13:14:15.123 | 13:14:15 |",
+        "+--------------------+-----------------+--------------+----------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // Fallible cases
+
+    let sql = "SELECT TIME 'not a time' as time;";
+    let result = try_execute_to_batches(&ctx, sql).await;
+    assert_eq!(
+        result.err().unwrap().to_string(),
+        "Arrow error: Cast error: Cannot cast string 'not a time' to value of Time64(Nanosecond) type"
+    );
+
+    // An invalid time
+    let sql = "SELECT TIME '24:01:02' as time;";
+    let result = try_execute_to_batches(&ctx, sql).await;
+    assert_eq!(
+        result.err().unwrap().to_string(),
+        "Arrow error: Cast error: Cannot cast string '24:01:02' to value of Time64(Nanosecond) type"
+    );
+}
+
+#[tokio::test]
+async fn cast_to_timestamp_twice() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp(a) from (select to_timestamp(1) as a)A;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+-------------------------------+",
+        "| totimestamp(a.a)              |",
+        "+-------------------------------+",
+        "| 1970-01-01 00:00:00.000000001 |",
+        "+-------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn cast_to_timestamp_seconds_twice() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql =
+        "select to_timestamp_seconds(a) from (select to_timestamp_seconds(1) as a)A;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+-------------------------+",
+        "| totimestampseconds(a.a) |",
+        "+-------------------------+",
+        "| 1970-01-01 00:00:01     |",
+        "+-------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn cast_to_timestamp_millis_twice() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp_millis(a) from (select to_timestamp_millis(1) as a)A;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+-------------------------+",
+        "| totimestampmillis(a.a)  |",
+        "+-------------------------+",
+        "| 1970-01-01 00:00:00.001 |",
+        "+-------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn cast_to_timestamp_micros_twice() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp_micros(a) from (select to_timestamp_micros(1) as a)A;";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+----------------------------+",
+        "| totimestampmicros(a.a)     |",
+        "+----------------------------+",
+        "| 1970-01-01 00:00:00.000001 |",
+        "+----------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn to_timestamp_i32() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp(cast (1 as int));";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+--------------------------------------+",
+        "| totimestamp(CAST(Int64(1) AS Int32)) |",
+        "+--------------------------------------+",
+        "| 1970-01-01 00:00:00.000000001        |",
+        "+--------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn to_timestamp_micros_i32() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp_micros(cast (1 as int));";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+--------------------------------------------+",
+        "| totimestampmicros(CAST(Int64(1) AS Int32)) |",
+        "+--------------------------------------------+",
+        "| 1970-01-01 00:00:00.000001                 |",
+        "+--------------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn to_timestamp_millis_i32() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp_millis(cast (1 as int));";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+--------------------------------------------+",
+        "| totimestampmillis(CAST(Int64(1) AS Int32)) |",
+        "+--------------------------------------------+",
+        "| 1970-01-01 00:00:00.001                    |",
+        "+--------------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn to_timestamp_seconds_i32() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "select to_timestamp_seconds(cast (1 as int));";
+    let results = execute_to_batches(&ctx, sql).await;
+
+    let expected = vec![
+        "+---------------------------------------------+",
+        "| totimestampseconds(CAST(Int64(1) AS Int32)) |",
+        "+---------------------------------------------+",
+        "| 1970-01-01 00:00:01                         |",
+        "+---------------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &results);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn date_bin() {
+    let ctx = SessionContext::new();
+
+    let sql = "SELECT DATE_BIN(INTERVAL '15 minutes', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '1970-01-01T00:00:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-08-03 14:30:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // Shift forward by 5 minutes
+    let sql = "SELECT DATE_BIN(INTERVAL '15 minutes', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '1970-01-01T00:05:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-08-03 14:35:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // Shift backward by 5 minutes
+    let sql = "SELECT DATE_BIN(INTERVAL '15 minutes', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '1970-01-01T23:55:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-08-03 14:25:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // origin after source, timestamp in previous bucket
+    let sql = "SELECT DATE_BIN(INTERVAL '15 minutes', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '2022-08-03 14:40:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-08-03 14:25:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // stride by 7 days
+    let sql = "SELECT DATE_BIN(INTERVAL '7 days', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '1970-01-01 00:00:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-07-28 00:00:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // origin shifts bins forward 1 day
+    let sql = "SELECT DATE_BIN(INTERVAL '7 days', TIMESTAMP '2022-08-03 14:38:50Z', TIMESTAMP '1970-01-02 00:00:00Z') AS res";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+",
+        "| res                 |",
+        "+---------------------+",
+        "| 2022-07-29 00:00:00 |",
+        "+---------------------+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // following test demonstrates array values for the source argument
+    let sql = "SELECT
+      DATE_BIN(INTERVAL '15' minute, time, TIMESTAMP '2001-01-01T00:00:00Z') AS time,
+      val
+    FROM (
+      VALUES
+        (TIMESTAMP '2021-06-10 17:05:00Z', 0.5),
+        (TIMESTAMP '2021-06-10 17:19:10Z', 0.3)
+      ) as t (time, val)";
+    let results = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------+-----+",
+        "| time                | val |",
+        "+---------------------+-----+",
+        "| 2021-06-10 17:00:00 | 0.5 |",
+        "| 2021-06-10 17:15:00 | 0.3 |",
+        "+---------------------+-----+",
+    ];
+    assert_batches_eq!(expected, &results);
+
+    // following test demonstrates array values for the origin argument are not currently supported
+    let sql = "SELECT
+      DATE_BIN(INTERVAL '15' minute, time, origin) AS time,
+      val
+    FROM (
+      VALUES
+        (TIMESTAMP '2021-06-10 17:05:00Z', TIMESTAMP '2001-01-01T00:00:00Z', 0.5),
+        (TIMESTAMP '2021-06-10 17:19:10Z', TIMESTAMP '2001-01-01T00:00:00Z', 0.3)
+      ) as t (time, origin, val)";
+    let result = try_execute_to_batches(&ctx, sql).await;
+    assert_eq!(
+        result.err().unwrap().to_string(),
+        "Arrow error: External error: This feature is not implemented: DATE_BIN only supports literal values for the origin argument, not arrays"
+    );
+}
+
+#[tokio::test]
+async fn timestamp_add_interval_second() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "SELECT NOW(), NOW() + INTERVAL '1' SECOND;";
+    let results = execute_to_batches(&ctx, sql).await;
+    let actual = result_vec(&results);
+
+    let res1 = actual[0][0].as_str();
+    let res2 = actual[0][1].as_str();
+
+    let format = "%Y-%m-%d %H:%M:%S%.6f";
+    let t1_naive = chrono::NaiveDateTime::parse_from_str(res1, format).unwrap();
+    let t2_naive = chrono::NaiveDateTime::parse_from_str(res2, format).unwrap();
+
+    assert_eq!(t1_naive.add(Duration::seconds(1)), t2_naive);
+    Ok(())
+}
+
+#[tokio::test]
+async fn timestamp_sub_interval_days() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "SELECT NOW(), NOW() - INTERVAL '8' DAY;";
+    let results = execute_to_batches(&ctx, sql).await;
+    let actual = result_vec(&results);
+
+    let res1 = actual[0][0].as_str();
+    let res2 = actual[0][1].as_str();
+
+    let format = "%Y-%m-%d %H:%M:%S%.6f";
+    let t1_naive = chrono::NaiveDateTime::parse_from_str(res1, format).unwrap();
+    let t2_naive = chrono::NaiveDateTime::parse_from_str(res2, format).unwrap();
+
+    assert_eq!(t1_naive.sub(Duration::days(8)), t2_naive);
+    Ok(())
+}
+
+#[tokio::test]
+async fn timestamp_add_interval_months() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "SELECT NOW(), NOW() + INTERVAL '4' MONTH;";
+    let results = execute_to_batches(&ctx, sql).await;
+    let actual = result_vec(&results);
+
+    let res1 = actual[0][0].as_str();
+    let res2 = actual[0][1].as_str();
+
+    let format = "%Y-%m-%d %H:%M:%S%.6f";
+    let t1_naive = chrono::NaiveDateTime::parse_from_str(res1, format).unwrap();
+    let t2_naive = chrono::NaiveDateTime::parse_from_str(res2, format).unwrap();
+
+    assert_eq!(t1_naive.with_month(t1_naive.month() + 4).unwrap(), t2_naive);
+    Ok(())
+}
+
+#[tokio::test]
+async fn timestamp_sub_interval_years() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    let sql = "SELECT NOW(), NOW() - INTERVAL '16' YEAR;";
+    let results = execute_to_batches(&ctx, sql).await;
+    let actual = result_vec(&results);
+
+    let res1 = actual[0][0].as_str();
+    let res2 = actual[0][1].as_str();
+
+    let format = "%Y-%m-%d %H:%M:%S%.6f";
+    let t1_naive = chrono::NaiveDateTime::parse_from_str(res1, format).unwrap();
+    let t2_naive = chrono::NaiveDateTime::parse_from_str(res2, format).unwrap();
+
+    assert_eq!(t1_naive.with_year(t1_naive.year() - 16).unwrap(), t2_naive);
+    Ok(())
+}
+
+#[tokio::test]
+async fn timestamp_array_add_interval() -> Result<()> {
+    let ctx = SessionContext::new();
+    let table_a = make_timestamp_table::<TimestampNanosecondType>()?;
+    let table_b = make_timestamp_table::<TimestampMicrosecondType>()?;
+    ctx.register_table("table_a", table_a)?;
+    ctx.register_table("table_b", table_b)?;
+
+    let sql = "SELECT ts, ts - INTERVAL '8' MILLISECONDS FROM table_a";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----------------------------+---------------------------------------+",
+        "| ts                         | table_a.ts Minus IntervalDayTime(\"8\") |",
+        "+----------------------------+---------------------------------------+",
+        "| 2020-09-08 13:42:29.190855 | 2020-09-08 13:42:29.182855            |",
+        "| 2020-09-08 12:42:29.190855 | 2020-09-08 12:42:29.182855            |",
+        "| 2020-09-08 11:42:29.190855 | 2020-09-08 11:42:29.182855            |",
+        "+----------------------------+---------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+
+    let sql = "SELECT ts, ts + INTERVAL '1' SECOND FROM table_b";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----------------------------+-----------------------------------------+",
+        "| ts                         | table_b.ts Plus IntervalDayTime(\"1000\") |",
+        "+----------------------------+-----------------------------------------+",
+        "| 2020-09-08 13:42:29.190855 | 2020-09-08 13:42:30.190855              |",
+        "| 2020-09-08 12:42:29.190855 | 2020-09-08 12:42:30.190855              |",
+        "| 2020-09-08 11:42:29.190855 | 2020-09-08 11:42:30.190855              |",
+        "+----------------------------+-----------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+
+    let sql = "SELECT ts, ts + INTERVAL '2' MONTH FROM table_b";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----------------------------+----------------------------------------+",
+        "| ts                         | table_b.ts Plus IntervalYearMonth(\"2\") |",
+        "+----------------------------+----------------------------------------+",
+        "| 2020-09-08 13:42:29.190855 | 2020-11-08 13:42:29.190855             |",
+        "| 2020-09-08 12:42:29.190855 | 2020-11-08 12:42:29.190855             |",
+        "| 2020-09-08 11:42:29.190855 | 2020-11-08 11:42:29.190855             |",
+        "+----------------------------+----------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+
+    let sql = "SELECT ts, ts - INTERVAL '16' YEAR FROM table_b";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----------------------------+-------------------------------------------+",
+        "| ts                         | table_b.ts Minus IntervalYearMonth(\"192\") |",
+        "+----------------------------+-------------------------------------------+",
+        "| 2020-09-08 13:42:29.190855 | 2004-09-08 13:42:29.190855                |",
+        "| 2020-09-08 12:42:29.190855 | 2004-09-08 12:42:29.190855                |",
+        "| 2020-09-08 11:42:29.190855 | 2004-09-08 11:42:29.190855                |",
+        "+----------------------------+-------------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
+#[tokio::test]
+async fn cast_timestamp_before_1970() -> Result<()> {
+    // this is a repro for issue #3082
+    let ctx = SessionContext::new();
+
+    let sql = "select cast('1969-01-01T00:00:00Z' as timestamp);";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+-------------------------------------------------------------------+",
+        "| CAST(Utf8(\"1969-01-01T00:00:00Z\") AS Timestamp(Nanosecond, None)) |",
+        "+-------------------------------------------------------------------+",
+        "| 1969-01-01 00:00:00                                               |",
+        "+-------------------------------------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &actual);
+
+    let sql = "select cast('1969-01-01T00:00:00.1Z' as timestamp);";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+---------------------------------------------------------------------+",
+        "| CAST(Utf8(\"1969-01-01T00:00:00.1Z\") AS Timestamp(Nanosecond, None)) |",
+        "+---------------------------------------------------------------------+",
+        "| 1969-01-01 00:00:00.100                                             |",
+        "+---------------------------------------------------------------------+",
+    ];
+
+    assert_batches_eq!(expected, &actual);
+
+    Ok(())
+}
