@@ -71,6 +71,10 @@ pub enum Operator {
     BitwiseAnd,
     /// Bitwise or, like `|`
     BitwiseOr,
+    /// Bitwise right, like `>>`
+    BitwiseShiftRight,
+    /// Bitwise left, like `<<`
+    BitwiseShiftLeft,
     /// String concat
     StringConcat,
 }
@@ -101,6 +105,8 @@ impl fmt::Display for Operator {
             Operator::IsNotDistinctFrom => "IS NOT DISTINCT FROM",
             Operator::BitwiseAnd => "&",
             Operator::BitwiseOr => "|",
+            Operator::BitwiseShiftRight => ">>",
+            Operator::BitwiseShiftLeft => "<<",
             Operator::StringConcat => "||",
         };
         write!(f, "{}", display)
