@@ -244,13 +244,23 @@ pub fn create_aggregate_expr(
             ));
         }
         (AggregateFunction::ApproxMedian, false) => {
-            Arc::new(expressions::ApproxMedian::new(
+            Arc::new(expressions::ApproxMedian::try_new(
                 coerced_phy_exprs[0].clone(),
                 name,
                 return_type,
-            ))
+            )?)
         }
         (AggregateFunction::ApproxMedian, true) => {
+            return Err(DataFusionError::NotImplemented(
+                "APPROX_MEDIAN(DISTINCT) aggregations are not available".to_string(),
+            ));
+        }
+        (AggregateFunction::Median, false) => Arc::new(expressions::Median::new(
+            coerced_phy_exprs[0].clone(),
+            name,
+            return_type,
+        )),
+        (AggregateFunction::Median, true) => {
             return Err(DataFusionError::NotImplemented(
                 "MEDIAN(DISTINCT) aggregations are not available".to_string(),
             ));
@@ -282,7 +292,7 @@ mod tests {
             DataType::Int32,
             DataType::Float32,
             DataType::Float64,
-            DataType::Decimal(10, 2),
+            DataType::Decimal128(10, 2),
             DataType::Utf8,
         ];
         for fun in funcs {
@@ -443,7 +453,7 @@ mod tests {
             DataType::Int32,
             DataType::Float32,
             DataType::Float64,
-            DataType::Decimal(10, 2),
+            DataType::Decimal128(10, 2),
             DataType::Utf8,
         ];
         for fun in funcs {
@@ -888,7 +898,7 @@ mod tests {
 
         let observed = return_type(
             &AggregateFunction::ApproxMedian,
-            &[DataType::Decimal(10, 6)],
+            &[DataType::Decimal128(10, 6)],
         );
         assert!(observed.is_err());
 
@@ -904,13 +914,14 @@ mod tests {
         assert_eq!(DataType::Int32, observed);
 
         // test decimal for min
-        let observed = return_type(&AggregateFunction::Min, &[DataType::Decimal(10, 6)])?;
-        assert_eq!(DataType::Decimal(10, 6), observed);
+        let observed =
+            return_type(&AggregateFunction::Min, &[DataType::Decimal128(10, 6)])?;
+        assert_eq!(DataType::Decimal128(10, 6), observed);
 
         // test decimal for max
         let observed =
-            return_type(&AggregateFunction::Max, &[DataType::Decimal(28, 13)])?;
-        assert_eq!(DataType::Decimal(28, 13), observed);
+            return_type(&AggregateFunction::Max, &[DataType::Decimal128(28, 13)])?;
+        assert_eq!(DataType::Decimal128(28, 13), observed);
 
         Ok(())
     }
@@ -929,11 +940,13 @@ mod tests {
         let observed = return_type(&AggregateFunction::Sum, &[DataType::Float64])?;
         assert_eq!(DataType::Float64, observed);
 
-        let observed = return_type(&AggregateFunction::Sum, &[DataType::Decimal(10, 5)])?;
-        assert_eq!(DataType::Decimal(20, 5), observed);
+        let observed =
+            return_type(&AggregateFunction::Sum, &[DataType::Decimal128(10, 5)])?;
+        assert_eq!(DataType::Decimal128(20, 5), observed);
 
-        let observed = return_type(&AggregateFunction::Sum, &[DataType::Decimal(35, 5)])?;
-        assert_eq!(DataType::Decimal(38, 5), observed);
+        let observed =
+            return_type(&AggregateFunction::Sum, &[DataType::Decimal128(35, 5)])?;
+        assert_eq!(DataType::Decimal128(38, 5), observed);
 
         Ok(())
     }
@@ -960,7 +973,7 @@ mod tests {
         assert_eq!(DataType::Int64, observed);
 
         let observed =
-            return_type(&AggregateFunction::Count, &[DataType::Decimal(28, 13)])?;
+            return_type(&AggregateFunction::Count, &[DataType::Decimal128(28, 13)])?;
         assert_eq!(DataType::Int64, observed);
         Ok(())
     }
@@ -976,11 +989,13 @@ mod tests {
         let observed = return_type(&AggregateFunction::Avg, &[DataType::Int32])?;
         assert_eq!(DataType::Float64, observed);
 
-        let observed = return_type(&AggregateFunction::Avg, &[DataType::Decimal(10, 6)])?;
-        assert_eq!(DataType::Decimal(14, 10), observed);
+        let observed =
+            return_type(&AggregateFunction::Avg, &[DataType::Decimal128(10, 6)])?;
+        assert_eq!(DataType::Decimal128(14, 10), observed);
 
-        let observed = return_type(&AggregateFunction::Avg, &[DataType::Decimal(36, 6)])?;
-        assert_eq!(DataType::Decimal(38, 10), observed);
+        let observed =
+            return_type(&AggregateFunction::Avg, &[DataType::Decimal128(36, 6)])?;
+        assert_eq!(DataType::Decimal128(38, 10), observed);
         Ok(())
     }
 
