@@ -28,7 +28,7 @@ use crate::expressions::format_state_name;
 use crate::{AggregateExpr, PhysicalExpr};
 use datafusion_common::ScalarValue;
 use datafusion_common::{DataFusionError, Result};
-use datafusion_expr::Accumulator;
+use datafusion_expr::{Accumulator, AggregateState};
 
 #[derive(Debug, PartialEq, Eq, Hash, Clone)]
 struct DistinctScalarValues(Vec<ScalarValue>);
@@ -54,7 +54,7 @@ impl DistinctCount {
         name: String,
         data_type: DataType,
     ) -> Self {
-        let state_data_types = input_data_types.into_iter().map(state_type).collect();
+        let state_data_types = input_data_types;
 
         Self {
             name,
@@ -65,15 +65,6 @@ impl DistinctCount {
     }
 }
 
-/// return the type to use to accumulate state for the specified input type
-fn state_type(data_type: DataType) -> DataType {
-    match data_type {
-        // when aggregating dictionary values, use the underlying value type
-        DataType::Dictionary(_key_type, value_type) => *value_type,
-        t => t,
-    }
-}
-
 impl AggregateExpr for DistinctCount {
     /// Return a reference to Any that can be used for downcasting
     fn as_any(&self) -> &dyn Any {
@@ -186,14 +177,16 @@ impl Accumulator for DistinctCountAccumulator {
             self.merge(&v)
         })
     }
-    fn state(&self) -> Result<Vec<ScalarValue>> {
+    fn state(&self) -> Result<Vec<AggregateState>> {
         let mut cols_out = self
             .state_data_types
             .iter()
             .map(|state_data_type| {
                 let values = Box::new(Vec::new());
-                let data_type = Box::new(state_data_type.clone());
-                ScalarValue::List(Some(*values), data_type)
+                ScalarValue::List(
+                    Some(*values),
+                    Box::new(Field::new("item", state_data_type.clone(), true)),
+                )
             })
             .collect::<Vec<_>>();
 
@@ -213,7 +206,7 @@ impl Accumulator for DistinctCountAccumulator {
             )
         });
 
-        Ok(cols_out)
+        Ok(cols_out.into_iter().map(AggregateState::Scalar).collect())
     }
 
     fn evaluate(&self) -> Result<ScalarValue> {
@@ -230,6 +223,7 @@ impl Accumulator for DistinctCountAccumulator {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::aggregate::utils::get_accum_scalar_values;
     use arrow::array::{
         ArrayRef, BooleanArray, Float32Array, Float64Array, Int16Array, Int32Array,
         Int64Array, Int8Array, ListArray, UInt16Array, UInt32Array, UInt64Array,
@@ -241,7 +235,7 @@ mod tests {
     macro_rules! state_to_vec {
         ($LIST:expr, $DATA_TYPE:ident, $PRIM_TY:ty) => {{
             match $LIST {
-                ScalarValue::List(_, data_type) => match data_type.as_ref() {
+                ScalarValue::List(_, field) => match field.data_type() {
                     &DataType::$DATA_TYPE => (),
                     _ => panic!("Unexpected DataType for list"),
                 },
@@ -274,22 +268,20 @@ mod tests {
                     Some(values) => {
                         for value in values.iter() {
                             match value {
-                                Some(v) => builder.values().append_value((*v).into())?,
-                                None => builder.values().append_null()?,
+                                Some(v) => builder.values().append_value((*v).into()),
+                                None => builder.values().append_null(),
                             }
                         }
 
-                        builder.append(true)?;
+                        builder.append(true);
                     }
                     None => {
-                        builder.append(false)?;
+                        builder.append(false);
                     }
                 }
             }
 
-            let array = Arc::new(builder.finish()) as ArrayRef;
-
-            Ok(array) as Result<ArrayRef>
+            Arc::new(builder.finish()) as ArrayRef
         }};
     }
 
@@ -350,7 +342,7 @@ mod tests {
         let mut accum = agg.create_accumulator()?;
         accum.update_batch(arrays)?;
 
-        Ok((accum.state()?, accum.evaluate()?))
+        Ok((get_accum_scalar_values(accum.as_ref())?, accum.evaluate()?))
     }
 
     fn run_update(
@@ -381,7 +373,7 @@ mod tests {
 
         accum.update_batch(&arrays)?;
 
-        Ok((accum.state()?, accum.evaluate()?))
+        Ok((get_accum_scalar_values(accum.as_ref())?, accum.evaluate()?))
     }
 
     fn run_merge_batch(arrays: &[ArrayRef]) -> Result<(Vec<ScalarValue>, ScalarValue)> {
@@ -399,7 +391,7 @@ mod tests {
         let mut accum = agg.create_accumulator()?;
         accum.merge_batch(arrays)?;
 
-        Ok((accum.state()?, accum.evaluate()?))
+        Ok((get_accum_scalar_values(accum.as_ref())?, accum.evaluate()?))
     }
 
     // Used trait to create associated constant for f32 and f64
@@ -703,7 +695,7 @@ mod tests {
                 Some(vec![Some(-2_i32), Some(-3_i32)]),
             ],
             Int32Builder
-        )?;
+        );
 
         let state_in2 = build_list!(
             vec![
@@ -711,7 +703,7 @@ mod tests {
                 Some(vec![Some(5_u64), Some(7_u64)]),
             ],
             UInt64Builder
-        )?;
+        );
 
         let (states, result) = run_merge_batch(&[state_in1, state_in2])?;
 
