@@ -155,9 +155,8 @@ mod tests {
                     None => builder.append_null(),
                     Some(v) => builder.append_value(v),
                 }
-                .unwrap()
             }
-            lb.append(true).unwrap();
+            lb.append(true);
         }
 
         lb.finish()
@@ -271,9 +270,8 @@ mod tests {
             match int_value {
                 None => fb.append_null(),
                 Some(v) => fb.append_value(v),
-            }
-            .unwrap();
-            builder.append(true).unwrap();
+            };
+            builder.append(true);
             let lb = builder
                 .field_builder::<ListBuilder<StringBuilder>>(1)
                 .unwrap();
@@ -281,10 +279,9 @@ mod tests {
                 match str_value {
                     None => lb.values().append_null(),
                     Some(v) => lb.values().append_value(v),
-                }
-                .unwrap();
+                };
             }
-            lb.append(true).unwrap();
+            lb.append(true);
         }
         builder.finish()
     }
