@@ -152,6 +152,32 @@ pub fn create_physical_expr(
                 }
             }
         }),
+        BuiltinScalarFunction::FromUnixtime => Arc::new({
+            match coerced_phy_exprs[0].data_type(input_schema) {
+                Ok(DataType::Int64) => |col_values: &[ColumnarValue]| {
+                    cast_column(
+                        &col_values[0],
+                        &DataType::Timestamp(TimeUnit::Second, None),
+                        &DEFAULT_DATAFUSION_CAST_OPTIONS,
+                    )
+                },
+                other => {
+                    return Err(DataFusionError::Internal(format!(
+                        "Unsupported data type {:?} for function from_unixtime",
+                        other,
+                    )))
+                }
+            }
+        }),
+        BuiltinScalarFunction::ArrowTypeof => {
+            let input_data_type = coerced_phy_exprs[0].data_type(input_schema)?;
+            Arc::new(move |_| {
+                Ok(ColumnarValue::Scalar(ScalarValue::Utf8(Some(format!(
+                    "{}",
+                    input_data_type
+                )))))
+            })
+        }
         // These don't need args and input schema
         _ => create_physical_fun(fun, execution_props)?,
     };
@@ -291,6 +317,9 @@ pub fn create_physical_fun(
         BuiltinScalarFunction::Power => {
             Arc::new(|args| make_scalar_function(math_expressions::power)(args))
         }
+        BuiltinScalarFunction::Atan2 => {
+            Arc::new(|args| make_scalar_function(math_expressions::atan2)(args))
+        }
 
         // string functions
         BuiltinScalarFunction::Array => Arc::new(array_expressions::array),
@@ -365,6 +394,7 @@ pub fn create_physical_fun(
         }
         BuiltinScalarFunction::DatePart => Arc::new(datetime_expressions::date_part),
         BuiltinScalarFunction::DateTrunc => Arc::new(datetime_expressions::date_trunc),
+        BuiltinScalarFunction::DateBin => Arc::new(datetime_expressions::date_bin),
         BuiltinScalarFunction::Now => {
             // bind value for now at plan time
             Arc::new(datetime_expressions::make_now(
@@ -1624,7 +1654,7 @@ mod tests {
         test_function!(
             Reverse,
             &[lit("loẅks")],
-            Ok(Some("skẅol")),
+            Ok(Some("sk̈wol")),
             &str,
             Utf8,
             StringArray
@@ -1633,7 +1663,7 @@ mod tests {
         test_function!(
             Reverse,
             &[lit("loẅks")],
-            Ok(Some("skẅol")),
+            Ok(Some("sk̈wol")),
             &str,
             Utf8,
             StringArray
