@@ -107,6 +107,19 @@ pub fn logical_plan_to_bytes(plan: &LogicalPlan) -> Result<Bytes> {
     logical_plan_to_bytes_with_extension_codec(plan, &extension_codec)
 }
 
+/// Serialize a LogicalPlan as json
+#[cfg(feature = "json")]
+pub fn logical_plan_to_json(plan: &LogicalPlan) -> Result<String> {
+    let extension_codec = DefaultExtensionCodec {};
+    let protobuf =
+        protobuf::LogicalPlanNode::try_from_logical_plan(plan, &extension_codec)
+            .map_err(|e| {
+                DataFusionError::Plan(format!("Error serializing plan: {}", e))
+            })?;
+    serde_json::to_string(&protobuf)
+        .map_err(|e| DataFusionError::Plan(format!("Error serializing plan: {}", e)))
+}
+
 /// Serialize a LogicalPlan as bytes, using the provided extension codec
 pub fn logical_plan_to_bytes_with_extension_codec(
     plan: &LogicalPlan,
@@ -121,6 +134,14 @@ pub fn logical_plan_to_bytes_with_extension_codec(
     Ok(buffer.into())
 }
 
+/// Deserialize a LogicalPlan from json
+#[cfg(feature = "json")]
+pub fn logical_plan_from_json(json: &str, ctx: &SessionContext) -> Result<LogicalPlan> {
+    let back: protobuf::LogicalPlanNode = serde_json::from_str(json).unwrap();
+    let extension_codec = DefaultExtensionCodec {};
+    back.try_into_logical_plan(ctx, &extension_codec)
+}
+
 /// Deserialize a LogicalPlan from bytes
 pub fn logical_plan_from_bytes(
     bytes: &[u8],
@@ -183,6 +204,31 @@ mod test {
         Expr::from_bytes(b"Leet").unwrap();
     }
 
+    #[test]
+    #[cfg(feature = "json")]
+    fn plan_to_json() {
+        use datafusion_common::DFSchema;
+        use datafusion_expr::logical_plan::EmptyRelation;
+
+        let plan = LogicalPlan::EmptyRelation(EmptyRelation {
+            produce_one_row: false,
+            schema: Arc::new(DFSchema::empty()),
+        });
+        let actual = logical_plan_to_json(&plan).unwrap();
+        let expected = r#"{"emptyRelation":{}}"#.to_string();
+        assert_eq!(actual, expected);
+    }
+
+    #[test]
+    #[cfg(feature = "json")]
+    fn json_to_plan() {
+        let input = r#"{"emptyRelation":{}}"#.to_string();
+        let ctx = SessionContext::new();
+        let actual = logical_plan_from_json(&input, &ctx).unwrap();
+        let result = matches!(actual, LogicalPlan::EmptyRelation(_));
+        assert!(result, "Should parse empty relation");
+    }
+
     #[test]
     fn udf_roundtrip_with_registry() {
         let ctx = context_with_udf();
