@@ -219,10 +219,13 @@ impl From<&DataType> for protobuf::arrow_type::ArrowTypeEnum {
                     value: Some(Box::new(value_type.as_ref().into())),
                 }))
             }
-            DataType::Decimal(whole, fractional) => Self::Decimal(protobuf::Decimal {
+            DataType::Decimal128(whole, fractional) => Self::Decimal(protobuf::Decimal {
                 whole: *whole as u64,
                 fractional: *fractional as u64,
             }),
+            DataType::Decimal256(_, _) => {
+                unimplemented!("The Decimal256 data type is not yet supported")
+            }
             DataType::Map(_, _) => {
                 unimplemented!("The Map data type is not yet supported")
             }
@@ -351,6 +354,7 @@ impl From<&AggregateFunction> for protobuf::AggregateFunction {
             }
             AggregateFunction::ApproxMedian => Self::ApproxMedian,
             AggregateFunction::Grouping => Self::Grouping,
+            AggregateFunction::Median => Self::Median,
         }
     }
 }
@@ -537,6 +541,7 @@ impl TryFrom<&Expr> for protobuf::LogicalExprNode {
                         protobuf::AggregateFunction::ApproxMedian
                     }
                     AggregateFunction::Grouping => protobuf::AggregateFunction::Grouping,
+                    AggregateFunction::Median => protobuf::AggregateFunction::Median,
                 };
 
                 let aggregate_expr = protobuf::AggregateExprNode {
@@ -835,25 +840,28 @@ impl TryFrom<&ScalarValue> for protobuf::ScalarValue {
                     Value::LargeUtf8Value(s.to_owned())
                 })
             }
-            scalar::ScalarValue::List(value, datatype) => {
-                println!("Current datatype of list: {:?}", datatype);
+            scalar::ScalarValue::List(value, boxed_field) => {
+                println!("Current field of list: {:?}", boxed_field);
                 match value {
                     Some(values) => {
                         if values.is_empty() {
                             protobuf::ScalarValue {
                                 value: Some(protobuf::scalar_value::Value::ListValue(
                                     protobuf::ScalarListValue {
-                                        datatype: Some(datatype.as_ref().try_into()?),
+                                        field: Some(boxed_field.as_ref().into()),
                                         values: Vec::new(),
                                     },
                                 )),
                             }
                         } else {
-                            let scalar_type = match datatype.as_ref() {
+                            let scalar_type = match boxed_field.data_type() {
                                 DataType::List(field) => field.as_ref().data_type(),
-                                _ => todo!("Proper error handling"),
+                                unsupported => {
+                                    todo!("Proper error handling {}", unsupported)
+                                }
                             };
                             println!("Current scalar type for list: {:?}", scalar_type);
+
                             let type_checked_values: Vec<protobuf::ScalarValue> = values
                                 .iter()
                                 .map(|scalar| match (scalar, scalar_type) {
@@ -862,7 +870,7 @@ impl TryFrom<&ScalarValue> for protobuf::ScalarValue {
                                         DataType::List(field),
                                     ) => {
                                         if let DataType::List(list_field) =
-                                            list_type.as_ref()
+                                            list_type.data_type()
                                         {
                                             let scalar_datatype = field.data_type();
                                             let list_datatype = list_field.data_type();
@@ -879,7 +887,8 @@ impl TryFrom<&ScalarValue> for protobuf::ScalarValue {
                                             scalar.try_into()
                                         } else {
                                             Err(Error::inconsistent_list_designated(
-                                                scalar, datatype,
+                                                scalar,
+                                                boxed_field.data_type(),
                                             ))
                                         }
                                     }
@@ -930,14 +939,15 @@ impl TryFrom<&ScalarValue> for protobuf::ScalarValue {
                                         DataType::LargeUtf8,
                                     ) => scalar.try_into(),
                                     _ => Err(Error::inconsistent_list_designated(
-                                        scalar, datatype,
+                                        scalar,
+                                        boxed_field.data_type(),
                                     )),
                                 })
                                 .collect::<Result<Vec<_>, _>>()?;
                             protobuf::ScalarValue {
                                 value: Some(protobuf::scalar_value::Value::ListValue(
                                     protobuf::ScalarListValue {
-                                        datatype: Some(datatype.as_ref().try_into()?),
+                                        field: Some(boxed_field.as_ref().into()),
                                         values: type_checked_values,
                                     },
                                 )),
@@ -946,7 +956,7 @@ impl TryFrom<&ScalarValue> for protobuf::ScalarValue {
                     }
                     None => protobuf::ScalarValue {
                         value: Some(protobuf::scalar_value::Value::NullListValue(
-                            datatype.as_ref().try_into()?,
+                            boxed_field.as_ref().try_into()?,
                         )),
                     },
                 }
@@ -1077,6 +1087,7 @@ impl TryFrom<&BuiltinScalarFunction> for protobuf::ScalarFunction {
             BuiltinScalarFunction::NullIf => Self::NullIf,
             BuiltinScalarFunction::DatePart => Self::DatePart,
             BuiltinScalarFunction::DateTrunc => Self::DateTrunc,
+            BuiltinScalarFunction::DateBin => Self::DateBin,
             BuiltinScalarFunction::MD5 => Self::Md5,
             BuiltinScalarFunction::SHA224 => Self::Sha224,
             BuiltinScalarFunction::SHA256 => Self::Sha256,
@@ -1115,17 +1126,20 @@ impl TryFrom<&BuiltinScalarFunction> for protobuf::ScalarFunction {
             BuiltinScalarFunction::Coalesce => Self::Coalesce,
             BuiltinScalarFunction::Power => Self::Power,
             BuiltinScalarFunction::Struct => Self::StructFun,
+            BuiltinScalarFunction::FromUnixtime => Self::FromUnixtime,
+            BuiltinScalarFunction::Atan2 => Self::Atan2,
+            BuiltinScalarFunction::ArrowTypeof => Self::ArrowTypeof,
         };
 
         Ok(scalar_function)
     }
 }
 
-impl TryFrom<&DataType> for protobuf::ScalarType {
+impl TryFrom<&Field> for protobuf::ScalarType {
     type Error = Error;
 
-    fn try_from(value: &DataType) -> Result<Self, Self::Error> {
-        let datatype = protobuf::scalar_type::Datatype::try_from(value)?;
+    fn try_from(value: &Field) -> Result<Self, Self::Error> {
+        let datatype = protobuf::scalar_type::Datatype::try_from(value.data_type())?;
         Ok(Self {
             datatype: Some(datatype),
         })
@@ -1231,7 +1245,8 @@ impl TryFrom<&DataType> for protobuf::scalar_type::Datatype {
             | DataType::Union(_, _, _)
             | DataType::Dictionary(_, _)
             | DataType::Map(_, _)
-            | DataType::Decimal(_, _) => {
+            | DataType::Decimal128(_, _)
+            | DataType::Decimal256(_, _) => {
                 return Err(Error::invalid_scalar_type(val));
             }
         };
